package edu.dhbw.brainbox;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.List;
import java.util.Properties;

import org.restlet.Component;
import org.restlet.Restlet;
import org.restlet.data.Protocol;
import org.restlet.resource.Directory;
import org.restlet.routing.Router;

import com.db4o.Db4oEmbedded;
import com.db4o.ObjectContainer;
import com.db4o.config.EmbeddedConfiguration;
import com.db4o.query.Predicate;

import edu.dhbw.brainbox.outputs.Powersocket;
import edu.dhbw.inputs.Sensor;
import edu.dhbw.resources.PowersocketResource;
import edu.dhbw.resources.SensorResource;
import edu.dhbw.resources.TriggerEventResource;
import edu.dhbw.triggers.BaseTrigger;

public class BrainApplication extends org.restlet.Application {
	// DB Container for db4o to persist objects (Powersocket) to disk.
	private final ObjectContainer container;
	private static Properties props = new Properties();
	private static Integer httpport = 8000;
	private static Integer maxThreads = 512;
	
	private TriggerRegistry triggerRegistry;

	public static void main(String... args) throws Exception {
		
        try {
                props.load(new FileInputStream("brainbox.properties"));
        } catch (IOException e) {
                System.err.println( "Settings property file not found. (Required for specify a port).\n");
        }
        
        //Config Part
        if (props.containsKey("port"))
        	httpport = Integer.parseInt(props.getProperty("port"));
        if (props.containsKey("maxThreads"))
        	maxThreads = Integer.parseInt(props.getProperty("maxThreads"));

		// Create a component with an HTTP server connector
		Component comp = new Component();
		comp.getServers().add(Protocol.HTTP, httpport);
		comp.getServers().get(0).getContext().getParameters().add("maxThreads", maxThreads.toString());
		
        //add file protocol for accessing static web files in some directories
        comp.getClients().add(Protocol.FILE);

		// Attach the application to the default host and start it
		comp.getDefaultHost().attach("/Brain", new BrainApplication());
		comp.start();
	}

	/**
	 * Constructor.
	 */
	public BrainApplication() {
		/** Open and keep the db4o object container. */
		EmbeddedConfiguration config = Db4oEmbedded.newConfiguration();
		config.common().updateDepth(5);
		this.container = Db4oEmbedded.openFile(config, "brain.dbo");
		
		triggerRegistry = loadRegistry();
		if (triggerRegistry == null)
			triggerRegistry = new TriggerRegistry("mainreg");
	}

	public Restlet createRoot() {
		Router router = new Router(getContext());
		// Add a route for powersocket resources
		router.attach("/powersocket/", PowersocketResource.class);
		router.attach("/powersocket", PowersocketResource.class);
//		router.attach("/powersocket/{powerid}", PowersocketResource.class);
		router.attach("/powersocket/{powerid}/", PowersocketResource.class);
		router.attach("/sensor/", SensorResource.class);
		router.attach("/sensor", SensorResource.class);
//		router.attach("/sensor/{sensorid}", SensorResource.class);
		router.attach("/sensor/{sensorid}/", SensorResource.class);
		router.attach("/trigger", TriggerEventResource.class);
		router.attach("/trigger/", TriggerEventResource.class);
//		router.attach("/trigger/{triggerid}", TriggerEventResource.class);
		router.attach("/trigger/{triggerid}/", TriggerEventResource.class);
		

		String dir = System.getProperty("user.dir");
		Directory webapp = new Directory(getContext(), "file://"+dir+"/WebContent/webapp/");
		webapp.setListingAllowed(false);
		webapp.setDeeplyAccessible(true);
        router.attach("/", webapp);
		return router;
	}

	/**
	 * Returns the database container.
	 * 
	 * @return the database container.
	 */
	public ObjectContainer getContainer() {
		return this.container;
	}

	private String powersocketsearchid;

	/*
	 * Query by Powersocket.powerid against db and returning the first object
	 * found.
	 */
	public Powersocket getPowersocketById(String powerid) {
		Powersocket result = null;
		powersocketsearchid = powerid;

		// Create the query predicate
		final Predicate<Powersocket> predicate = new Predicate<Powersocket>() {
			private static final long serialVersionUID = 1L;

			@Override
			public boolean match(Powersocket candidate) {
				return BrainApplication.this.powersocketsearchid
						.equals(candidate.getPowerid());
			}
		};

		// Query the database and get the first result
		final List<Powersocket> powersockets = getContainer().query(predicate);
		if ((powersockets != null) && (powersockets.size() > 0)) {
			result = powersockets.get(0);
		}

		return result;
	}
	
	/*
	 * Get All Powersockets
	 * 
	 */
	public List<Powersocket> getAllPowersockets() {
		return getContainer().queryByExample(new Powersocket(null, null, null));
	}

	/*
	 * Saving a Powersocket object (param) into DB.
	 */
	public void putPowersocket(Powersocket aPowersocket) {
		getContainer().store(aPowersocket);
		getContainer().commit();
		System.out.println("Powersocket saved: "+aPowersocket.getId());
	}

	// TODO: Predicate Query by Object instead of Sensor & Powersocket?
	private String sensorsearchid;

	/*
	 * Query by Sensor.sensorid against db and returning the first object found.
	 */
	public Sensor getSensorById(String sensorid) {
		Sensor result = null;
		sensorsearchid = sensorid;

		// Create the query predicate
		final Predicate<Sensor> predicate = new Predicate<Sensor>() {
			private static final long serialVersionUID = 1L;

			@Override
			public boolean match(Sensor candidate) {
				return BrainApplication.this.sensorsearchid.equals(candidate
						.getId());
			}
		};

		// Query the database and get the first result
		final List<Sensor> sensors = getContainer().query(predicate);
		if ((sensors != null) && (sensors.size() > 0)) {
			result = sensors.get(0);
		}

		return result;
	}

	/*
	 * Saving a Sensor object (param) into DB.
	 */
	public void saveSensor(Sensor sensor, Boolean valueChanged) {
			getContainer().store(sensor);
			getContainer().commit();
			System.out.println("Sensor saved: "+sensor.getId());
			
			if (valueChanged)
				triggerRegistry.checkForEvent(sensor);
	}

	/*
	 * Query by Sensor.sensorid against db and returning the first object found.
	 */
	public BaseTrigger getTriggerById(String triggerid) {
		BaseTrigger result = null;

		// Query the database and get the first result
		final List<BaseTrigger> triggers = getContainer().queryByExample(new BaseTrigger(triggerid));
		if ((triggers != null) && (triggers.size() > 0)) {
			result = triggers.get(0);
		}

		return result;
	}

	/*
	 * Saving a Sensor object (param) into DB.
	 */
	public void saveTrigger(BaseTrigger trigger) {
			getContainer().store(trigger);
			getContainer().commit();
			System.out.println("Saved Trigger: "+trigger.getId());
			
			triggerRegistry.registerTrigger(trigger);
			System.out.println("Registered Trigger: "+trigger.getId() + " in Registry.");
			saveRegistry();
	}
	
	private void saveRegistry() {
		getContainer().store(triggerRegistry);
		getContainer().commit();
	}
	
	private TriggerRegistry loadRegistry() {
		final List<TriggerRegistry> registry = getContainer().queryByExample(new TriggerRegistry("mainreg"));
		if (registry != null && registry.size() > 0)
			return registry.get(0);
		else
			return null;
	}
}
